/***************************************************************************
                          kvirus.cpp  -  description                              
                             -------------------                                         
    begin                : Sat Jun 12 00:50:22 PDT 1999
                                           
    copyright            : (C) 1999 by Sung Soo Kang                         
    email                : sskang@po-box.mcgill.ca                                     
 ***************************************************************************/
 /***************************************************************************
  The original vergion of Kvirus is Ataxxlet written in JAVA
    by Paul Buchheit <ptb@po.cwru.edu>
  Kvirus is KDE port of Ataxxlet.
****************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include <kvirus.h>
#include <kaudio.h>

KvirusApp::KvirusApp()
{
  setCaption("Kvirus " VERSION);

  ///////////////////////////////////////////////////////////////////
  // read the config file options
  readOptions();

  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initMenuBar();
  initToolBar();
  initStatusBar();
  initView();

  ///////////////////////////////////////////////////////////////////
  // enable bars dependend on config file setups
  if (!bViewToolbar)
    enableToolBar(KToolBar::Hide,0);
  if (!bViewStatusbar)
    enableStatusBar(KStatusBar::Hide);

  menuBar()->setMenuBarPos(menu_bar_pos);
  toolBar()->setBarPos(tool_bar_pos);

  ///////////////////////////////////////////////////////////////////
  // disable menu and toolbar items at startup
  this->resize(view->window_width, view->window_height);
}

KvirusApp::~KvirusApp()
{
  
}

void KvirusApp::enableCommand(int id_)
{
  ///////////////////////////////////////////////////////////////////
  // enable menu and toolbar functions by their ID's
  menuBar()->setItemEnabled(id_,true);
  toolBar()->setItemEnabled(id_,true);
}

void KvirusApp::disableCommand(int id_)
{
  ///////////////////////////////////////////////////////////////////
  // disable menu and toolbar functions by their ID's
  menuBar()->setItemEnabled(id_,false);
  toolBar()->setItemEnabled(id_,false);
}


void KvirusApp::initMenuBar()
{

  ///////////////////////////////////////////////////////////////////
  // MENUBAR  

  ///////////////////////////////////////////////////////////////////
  // menuBar entry game_menu
  game_menu = new QPopupMenu();
  game_menu->insertItem(Icon("mini/kvirus.xpm"), i18n("&New Game"), ID_GAME_NEW );
  game_menu->insertSeparator();
  game_menu->insertItem(i18n("E&xit"), ID_GAME_QUIT );

  game_menu->setAccel(CTRL+Key_N, ID_GAME_NEW);

  //////////////////////////////////////////////////////////////////
  //menuBar entry opeion_menu
  option_menu = new QPopupMenu();
  option_menu->setCheckable(TRUE);
  option_menu->insertItem(i18n("Sound"), ID_SOUND_MODE);
  option_menu->setItemChecked(ID_SOUND_MODE, TRUE);
  bSound=TRUE;
  option_menu->insertItem(i18n("Blocks"), ID_BLOCK_ENABLE);
  option_menu->setItemChecked(ID_BLOCK_ENABLE, TRUE);
  bViewBlock=TRUE;
  option_menu->insertSeparator();
  option_menu->insertItem(i18n("vs. Computer"), ID_SINGLE_MODE);
  option_menu->setItemChecked(ID_SINGLE_MODE, TRUE);
	option_menu->insertItem(i18n("vs. Human"), ID_MULTI_MODE);
	option_menu->setItemChecked(ID_MULTI_MODE, FALSE);
	option_menu->insertSeparator();
	
	size_menu=new QPopupMenu();
	size_menu->setCheckable(TRUE);
	size_menu->insertItem(i18n("7X7"), ID_SEVEN_BOARD);
	size_menu->setItemChecked(ID_SEVEN_BOARD, FALSE);
	size_menu->insertItem(i18n("9X9 Default"), ID_NINE_BOARD);
	size_menu->setItemChecked(ID_NINE_BOARD, TRUE);
	size_menu->insertItem(i18n("11X11"), ID_ELEVEN_BOARD);
	size_menu->setItemChecked(ID_ELEVEN_BOARD, FALSE);
	option_menu->insertItem(i18n("Board Size"), size_menu);
  ///////////////////////////////////////////////////////////////////
  // menuBar entry help_menu
  help_menu = new QPopupMenu();
  help_menu = kapp->getHelpMenu(true, i18n(IDS_APP_ABOUT));


  ///////////////////////////////////////////////////////////////////
  // MENUBAR CONFIGURATION
  menuBar()->insertItem(i18n("&Game"), game_menu);
  menuBar()->insertItem(i18n("&Option"), option_menu);
  ///////////////////////////////////////////////////////////////////
  // INSERT YOUR APPLICATION SPECIFIC MENUENTRIES HERE


  menuBar()->insertSeparator();
  menuBar()->insertItem(i18n("&Help"), help_menu);

  ///////////////////////////////////////////////////////////////////
  // CONNECT THE SUBMENU SLOTS WITH SIGNALS

  CONNECT_CMD(game_menu);
  CONNECT_CMD(option_menu);
  CONNECT_CMD(size_menu);

}
void KvirusApp::initToolBar()
{

  ///////////////////////////////////////////////////////////////////
  // TOOLBAR
  // set toolBar() the current toolBar and the position due to config file
  toolBar()->insertButton(Icon("mini/kvirus.xpm"), ID_GAME_NEW, true, i18n("New Game"));
  toolBar()->insertButton(Icon("help.xpm"), ID_HELP, SIGNAL(pressed()), kapp, SLOT(appHelpActivated()), true, i18n("Help"));

  ///////////////////////////////////////////////////////////////////
  // INSERT YOUR APPLICATION SPECIFIC TOOLBARS HERE -e.g. tool_bar_1:
  // add functionality for new created toolbars in:
  // enableCommand, disableCommand, in the menuBar() and an additional function slotViewToolbar_1
  // for that also create a bViewToolbar_1 and a KConfig entry (see Constructor).
  // Also update resource values and commands 


  ///////////////////////////////////////////////////////////////////
  // CONNECT THE TOOLBAR SLOTS WITH SIGNALS - add new created toolbars
  CONNECT_TOOLBAR(toolBar());

}

void KvirusApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  //STATUSBAR
  statusBar()->insertItem(IDS_DEFAULT, ID_STATUS_MSG );
  statusBar()->setInsertOrder(KStatusBar::RightToLeft);

}

void KvirusApp::initView()
{ 
  doc = new KvirusDoc(this);

  ////////////////////////////////////////////////////////////////////
  // set the main widget here
  KApplication *app=KApplication::getKApplication();
  view = new KvirusView(app,doc,this);
  setView(view);

}

bool KvirusApp::queryExit()
{
  int exit=KMsgBox::yesNo(this, i18n("Exit"), i18n("Really Quit ?"));

  if(exit==1)
    return true;
  else
    return false;
}

void KvirusApp::saveOptions()
{
  KConfig *config = kapp->getConfig();


  config->setGroup("APPEARANCE");
  config->writeEntry("ShowToolbar",toolBar()->isVisible());
  config->writeEntry("ShowStatusbar",statusBar()->isVisible());
  config->writeEntry("MenuBarPos", (int)menuBar()->menuBarPos());
  config->writeEntry("ToolBar_Pos", (int)toolBar()->barPos());

}

void KvirusApp::readOptions()
{
  ///////////////////////////////////////////////////////////////////
  // read the config file entries
  KConfig *config = kapp->getConfig();

 
  config->setGroup("APPEARANCE");
  bViewToolbar = config->readBoolEntry("ShowToolbar", true);
  bViewStatusbar = config->readBoolEntry("ShowStatusbar", true);
  menu_bar_pos = (KMenuBar::menuPosition)config->readNumEntry("MenuBarPos", KMenuBar::Top); 
  tool_bar_pos = (KToolBar::BarPosition)config->readNumEntry("ToolBar_Pos", KToolBar::Top);

}


/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void KvirusApp::slotGameNew()
{
  slotStatusMsg(i18n("Creating new game..."));

  slotStatusMsg(IDS_DEFAULT);
  view->newGame();
  view->setGameFlag(0);
}





void KvirusApp::slotGameCloseWindow()
{
  close();
}
void KvirusApp::slotGameQuit()
{ 

  ///////////////////////////////////////////////////////////////////
  // exits the Application
  if(this->queryExit())
    {
      saveOptions();
      KTMainWindow::deleteAll();
      kapp->quit();
    }
  else
    slotStatusMsg(IDS_DEFAULT);
    return;
}


void KvirusApp::slotSound()
{
	slotStatusMsg(i18n("Sound Enable or Disable.."));
  slotStatusMsg(IDS_DEFAULT);
  if(bSound){
    option_menu->setItemChecked(ID_SOUND_MODE, FALSE);
    view->setSoundMode(0);
	}
  else{
    option_menu->setItemChecked(ID_SOUND_MODE, TRUE);
    view->setSoundMode(1);
  }
  bSound=!bSound;
}
	
void KvirusApp::slotBlock()
{
  slotStatusMsg(i18n("Block Enable or Disable.."));
  slotStatusMsg(IDS_DEFAULT);
  if(bViewBlock){
    option_menu->setItemChecked(ID_BLOCK_ENABLE, FALSE);
    view->setBlockFlag(0);
    view->newGame();
    view->setGameFlag(0);
  }
  else{
    option_menu->setItemChecked(ID_BLOCK_ENABLE, TRUE);
    view->setBlockFlag(1);
    view->newGame();
    view->setGameFlag(0);
  }
  bViewBlock=!bViewBlock;
}

void KvirusApp::slotSingle()
{
	slotStatusMsg(i18n("Play with computer.."));
	slotStatusMsg(IDS_DEFAULT);
	option_menu->setItemChecked(ID_SINGLE_MODE, TRUE);
	option_menu->setItemChecked(ID_MULTI_MODE, FALSE);
	view->setGamemode(0);
	view->newGame();
	view->setGameFlag(0);

}

void KvirusApp::slotMulti()
{
	slotStatusMsg(i18n("Multi play.."));
	slotStatusMsg(IDS_DEFAULT);
	option_menu->setItemChecked(ID_SINGLE_MODE, FALSE);
	option_menu->setItemChecked(ID_MULTI_MODE, TRUE);
	view->setGamemode(1);
	view->newGame();
	view->setGameFlag(0);

}


void KvirusApp::slotSize(int id){
	switch(id){
		case 7:
			size_menu->setItemChecked(ID_SEVEN_BOARD, TRUE);
			size_menu->setItemChecked(ID_NINE_BOARD, FALSE);
			size_menu->setItemChecked(ID_ELEVEN_BOARD, FALSE);
			view->dim=7;
			break;
		case 9:
			size_menu->setItemChecked(ID_SEVEN_BOARD, FALSE);
			size_menu->setItemChecked(ID_NINE_BOARD, TRUE);
			size_menu->setItemChecked(ID_ELEVEN_BOARD, FALSE);
			view->dim=9;
			break;
		case 11:
			size_menu->setItemChecked(ID_SEVEN_BOARD, FALSE);
			size_menu->setItemChecked(ID_NINE_BOARD, FALSE);
			size_menu->setItemChecked(ID_ELEVEN_BOARD, TRUE);
			view->dim=11;
			break;
	}
	view->newGame();
	view->setGameFlag(0);
	this->resize(33.5*view->dim+10, 39.5*view->dim+15);
	
}
void KvirusApp::slotStatusMsg(const char *text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message permanently
  statusBar()->clear();
  statusBar()->changeItem(text, ID_STATUS_MSG );
}


void KvirusApp::slotStatusHelpMsg(const char *text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message of whole statusbar temporary (text, msec)
  statusBar()->message(text, 2000);
}



void KvirusApp::commandCallback(int id_){
  switch (id_){
    ON_CMD(ID_GAME_NEW,                 slotGameNew())
    ON_CMD(ID_GAME_CLOSE_WINDOW,        slotGameCloseWindow())
    ON_CMD(ID_GAME_QUIT,                slotGameQuit())
		ON_CMD(ID_SOUND_MODE, 							slotSound())
    ON_CMD(ID_BLOCK_ENABLE,                    slotBlock())
    ON_CMD(ID_SINGLE_MODE, 	slotSingle())
    ON_CMD(ID_MULTI_MODE, slotMulti())
    ON_CMD(ID_SEVEN_BOARD, slotSize(7))
    ON_CMD(ID_NINE_BOARD, slotSize(9))
    ON_CMD(ID_ELEVEN_BOARD, slotSize(11))

  }
}

void KvirusApp::statusCallback(int id_){
  switch (id_){
    ON_STATUS_MSG(ID_GAME_NEW,          i18n("Creates a new game"))
    ON_STATUS_MSG(ID_GAME_CLOSE_WINDOW, i18n("Closes the current window"))
    ON_STATUS_MSG(ID_GAME_QUIT,         i18n("Exits the program"))
    ON_STATUS_MSG(ID_SOUND_MODE, 				i18n("Enable and distables sound"))
    ON_STATUS_MSG(ID_BLOCK_ENABLE,      i18n("Enable and disables block"))
    ON_STATUS_MSG(ID_SINGLE_MODE, 	i18n("Play Single Mode"))
		ON_STATUS_MSG(ID_MULTI_MODE, 	i18n("Play Multi Mode"))
  }
}























































